/* odbc-test-06.c: Sending data to the DBMS, with SQLBindParameter
 * This program compiled and run, without changes, on:
 *   Red Hat Linux 8.0, gcc 3.2, unixODBC 2.2.2
 *     ("gcc -Wall odbc-test-06.c -o odbc-test-06 -lodbc")
 *   Windows 2000 Professional, Microsoft Visual C++ 6.0
 *   Windows 2000 Professional, Borland C++ Builder 5.0
 * All of those with Mimer 9.1.3, and Mimer ODBC driver 9.something
 * Thomas Padron-McCarthy (Thomas.Padron-McCarthy@tech.oru.se)
 * May 25, 2004
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#if defined(_MSDOS) || defined(_WIN32)
#include <windows.h>
#endif
#include "sqlext.h"

int main(void) {
  SQLHENV eh;
  SQLHDBC ch;
  SQLHSTMT sh;
  SQLCHAR wanted_name[20 + 1];
  SQLINTEGER wanted_name_size;
  int n;
  SQLINTEGER wanted_number;
  SQLINTEGER wanted_number_size;

  /* ODBC 2.x uses SQLAllocEnv, SQLAllocConnect and SQLAllocStmt.
   * ODBC 3.x uses SQLAllocHandle, with different arguments, for all of those.
   * First: Allocate environment handle.
   */
  if (SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &eh) != SQL_SUCCESS) {
    fprintf(stderr, "Kunde inte allokera en ODBC-omgivning.\n");
    return EXIT_FAILURE;
  }

  /* Set the ODBC version attribute,
   * so ODBC knows which ODBC version this application was written for.
   */
  if (SQLSetEnvAttr(eh,
		    SQL_ATTR_ODBC_VERSION, (SQLPOINTER)SQL_OV_ODBC3,
		    SQL_IS_INTEGER) != SQL_SUCCESS) {
    fprintf(stderr, "Kunde inte stta ODBC-versionen till SQL_OV_ODBC3.\n");
    return EXIT_FAILURE;
  }

  /* Allocate connection handle */
  if (SQLAllocHandle(SQL_HANDLE_DBC, eh, &ch) != SQL_SUCCESS) {
    fprintf(stderr, "Kunde inte allokera ett anslutningsobjekt.\n");
    return EXIT_FAILURE;
  }

  /* Connect: data source, user name, password */
  if (SQLConnect(ch, (SQLCHAR*)"demobasen", SQL_NTS,
		 (SQLCHAR*)"demouser", SQL_NTS,
		 (SQLCHAR*)"fnord", SQL_NTS) != SQL_SUCCESS) {
    fprintf(stderr, "Kunde inte ansluta till datakllan 'demobasen'.\n");
    return EXIT_FAILURE;
  }

  /* Allocate statement handle */
  if (SQLAllocHandle(SQL_HANDLE_STMT, ch, &sh) != SQL_SUCCESS) {
    fprintf(stderr, "Kunde inte allokera ett statement handle.\n");
    return EXIT_FAILURE;
  }

  if (SQLExecDirect(sh, (SQLCHAR*)"select Nummer, Namn from Person", SQL_NTS)
      != SQL_SUCCESS) {
    fprintf(stderr, "Kunde inte kra frgan.\n");
    return EXIT_FAILURE;
  }

  printf("Personer:\n");
  while (SQLFetch(sh) == SQL_SUCCESS) {
    SQLINTEGER number;
    SQLCHAR name[20 + 1];
    SQLINTEGER number_size, name_size;

    SQLGetData(sh, 1, SQL_C_SLONG, &number, sizeof number, &number_size);
    SQLGetData(sh, 2, SQL_C_CHAR, name, sizeof name, &name_size);

    printf("Nummer %d, med namnet %s\n", (int)number, name);
  }
 
  /* You have to close the cursor before re-using the statement handle! */
  SQLCloseCursor(sh);

  printf("Ange namnet p en av personerna: ");
  fgets(wanted_name, sizeof wanted_name, stdin);
  if (wanted_name[strlen(wanted_name) - 1] == '\n')
    wanted_name[strlen(wanted_name) - 1] = '\0';
  printf("Du har valt namnet '%s'.\n", wanted_name);

  printf("Ange numret p en av personerna: ");
  if (scanf("%d", &n) != 1) {
    fprintf(stderr, "Det dr var inte ett korrekt formaterat heltal.\n");
    return EXIT_FAILURE;
  }
  /* Skip the rest of the line, after the integer */
  while (getchar() != '\n')
    ;
  /* We want to be sure that we use the correct data types,
     both for sscanf (int) and SQLBindParameter (SQL_C_SLONG). */
  wanted_number = n;
  printf("Du har valt numret %d.\n", (int)wanted_number);

  SQLBindParameter(sh, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR,
		   20, 0, wanted_name, sizeof wanted_name, &wanted_name_size);
  SQLBindParameter(sh, 2, SQL_PARAM_INPUT, SQL_C_SLONG, SQL_INTEGER,
		   0, 0, &wanted_number, sizeof wanted_number, &wanted_number_size);

  /* You have to put the string length, or SQL_NTS, in wanted_name_size! */
  wanted_name_size = SQL_NTS;
  /* And the size of the number too! Always give the size of each parameter! */
  wanted_number_size = sizeof wanted_number;

  if (SQLExecDirect(sh, (SQLCHAR*)"select Nummer, Namn from Person"
		                  " where Namn = ? or Nummer = ?", SQL_NTS) != SQL_SUCCESS) {
    fprintf(stderr, "SQL-frgan misslyckades.\n");
    return EXIT_FAILURE;
  }

  printf("De skta personerna:\n");
  while (SQLFetch(sh) == SQL_SUCCESS) {
    SQLINTEGER number;
    SQLCHAR name[20 + 1];
    SQLINTEGER number_size, name_size;

    SQLGetData(sh, 1, SQL_C_SLONG, &number, sizeof number, &number_size);
    SQLGetData(sh, 2, SQL_C_CHAR, name, sizeof name, &name_size);

    printf("Nummer %d, med namnet %s\n", (int)number, name);
  }

  SQLFreeHandle(SQL_HANDLE_STMT, sh);
  SQLDisconnect(ch);
  SQLFreeHandle(SQL_HANDLE_DBC, ch);
  SQLFreeHandle(SQL_HANDLE_ENV, eh);

  return EXIT_SUCCESS;
} /* main */
